/**
 * This class instantiates a simple Infix Calculator that reads input from
 * standard in and then computes and displays the answer or prints
 * an error message.
 *
 * @author <A HREF="mailto:gte220j@prism.gatech.edu.com"> J.Chris Holmes </A>
 * @version Version 1.0, 4/23/2001
 */

import java.util.StringTokenizer;
import java.io.IOException;

public class Infix_Calc 
{
  /**
   * Default constructor for an Infix Calculator object
   */
  public Infix_Calc() {
    boolean not_done = true;
    String input_string; 
    int output;

    while( not_done == true) {
      try {
        input_string = IOHelp.readLine();

        if(input_string.compareToIgnoreCase("q") == 0) 
          not_done = false;
        else { 
          try {
            output = RunCalculation(input_string);
            System.out.println("ans = "+output);
          }
          catch(InvalidInputException iie) {
            System.out.println(iie.getMessage());
          }
        }
      }
      catch(IOException ioe) {
        ioe.printStackTrace();
        not_done = false;
      }
    }
  }
  /**
   * Takes in a String of math in Infix Notation form (no parentheses) and
   * returns the answer or throws an Exception.
   *
   * @param input_string  A string of math in Infix Notation form
   * @return  An int of the result
   */
  public int RunCalculation(String input_string) throws InvalidInputException {
    StringTokenizer strtok = new StringTokenizer(input_string, " \t");
    String token1, token2;
    int op1, op2;

    if(strtok.hasMoreTokens() == false) { 
      throw new InvalidInputException("ERROR:  Expression Invalid");
    }
    
    token1 = strtok.nextToken();
    if( isNum(token1) != true) {
      throw new InvalidInputException("ERROR:  Expression Invalid");
    }
    op1 = Integer.parseInt(token1);

    while( strtok.countTokens() >= 2) {
      token1 = strtok.nextToken();
      token2 = strtok.nextToken();

      if( isOperator(token1) != true) {
        throw new InvalidInputException("ERROR:  Expression Invalid");
      }
      if( isNum(token2) != true) {
        throw new InvalidInputException("ERROR:  Expression Invalid");
      }

      op2 = Integer.parseInt(token2);
 
      switch(token1.charAt(0)) {
        case '+': 
          op1 = op1 + op2;
         break;
        case '-': 
          op1 = op1 - op2;          
         break;
        case '*': 
          op1 = op1 * op2;
         break;
        case '/': 
          op1 = op1 / op2;
         break;
        case '^': 
          op1 = (int)Math.pow((double)op1,(double)op2);
         break;
        default: 
         break;
      }
    }
    if(strtok.hasMoreTokens() == true) {
      throw new InvalidInputException("ERROR:  Expression Invalid");
    }
    return(op1);
  } 
  /**
   * Takes in a single string token and returns a boolean based on
   * whether or not the string is a valid number
   *
   * @param to_check  Any String
   * @return  A boolean that is true is the input String is a valid number
   */
  public boolean isNum(String to_check) {
    boolean toret = false;
    try {
      Integer.parseInt(to_check);
      toret = true;
    }
    catch(NumberFormatException nfe) {
      toret = false;
    }
    return(toret);
  }
  /**
   * Takes in a single string token and returns a boolean based on
   * whether or not the string is a valid mathematical operator
   *
   * @param to_check  Any String
   * @return  A boolean that is true is the input String is a valid operator
   */
  public boolean isOperator(String to_check) {
    if(to_check == null) return false;
    if(to_check.length() != 1) return false; 

    boolean toret = false;
    switch(to_check.charAt(0)) {
      case '+': 
      case '-':
      case '*':
      case '/':
      case '^':
        toret = true; break;
      default: break;
    }
    return(toret);
  }
  /**
   * Main method -- instantiates a Calculator
   *
   * @param args  Command line arguements
   */
  public static void main(String args[]) {
    new Infix_Calc();
  }
}